//
// Copyright 1993 Massachusetts Institute of Technology
//
// Permission to use, copy, modify, distribute, and sell this software and
// its documentation for any purpose is hereby granted without fee, provided
// that the above copyright notice appear in all copies and that both that
// copyright notice and this permission notice appear in supporting
// documentation, and that the name of M.I.T. not be used in advertising or
// publicity pertaining to distribution of the software without specific,
// written prior permission.  M.I.T. makes no representations about the
// suitability of this software for any purpose.  It is provided "as is"
// without express or implied warranty.
//
// M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
// ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
// M.I.T.  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
// ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER IN AN ACTION * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
// SOFTWARE.
//
// Copyright (c) 1991, 1992 Redwood Design Automation
//
// Permission to use, copy, modify, distribute, and sell this software and
// its documentation for any purpose is hereby granted without fee, provided
// that (i) the above copyright notices and this permission notice appear in
// all copies of the software and related documentation, and (ii) the name of
// Redwood Design Automation may not be used in any advertising or publicity
// relating to the software without the specific, prior written permission of
// Redwood Design Automation.
//
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
// INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
// ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
// SOFTWARE.
//
// Author: Denis Bohm  Sep. 1992 
// Revised: Joerg Ringel Nov. 1992 
// Further abuse: Dave Sternlicht Dec. 1992
//                The name of this class was changed from ruiScene.
//

#ifndef tree_h
#define tree_h

#include <InterViews/adjust.h>
#include <InterViews/brush.h>
#include <InterViews/color.h>
#include <InterViews/glyph.h>
#include <InterViews/observe.h>
#include <InterViews/session.h>
#include <InterViews/transformer.h>
#include <IV-look/kit.h>
#include <OS/list.h>
#include <IV-X11/xevent.h>
#include "stdio.h"

#include <InterViews/_enter.h>

class TreeDepthList;
class TreeImpl;
class TreeInfo;
class TreeInfoPtrList;

//
// Tree members
//

typedef enum { TopDown, LeftToRight } Orientation;

class Tree : public Glyph, public Adjustable, public Observer {
public:
  Tree(Session*);
  virtual ~Tree();

  // Tree API Functions  
  void insert_node(Glyph *parent, Glyph* child);
  void force_layout(Orientation = LeftToRight);

  // Glyph
  void virtual request(Requisition&) const;
  void virtual allocate(Canvas*, const Allocation&, Extension&);
  void virtual draw(Canvas*, const Allocation&) const;
  void virtual pick(Canvas*, const Allocation&, int depth, Hit&);
  void virtual allotment(GlyphIndex, DimensionName, Allotment&) const;

  // Adjustable
  virtual Coord  upper(DimensionName) const;
  virtual Coord cur_lower(DimensionName) const;
  virtual Coord cur_upper(DimensionName) const;
  virtual Coord cur_length(DimensionName) const;
  virtual void scroll_to(DimensionName, Coord lower);
  virtual void scale_to(DimensionName, float fraction_visible);
  virtual void zoom_to(float magnification);

  // Observer
  void update(Observable*);
private:
  TreeImpl* timpl_; // you can't touch this :-)
};

#include <InterViews/_leave.h>

#endif

